
*	frontend.f
*	A modification of the original frontend.f program. There are 
*	16 detectors. The first 15 are band pass filters and the last 
*	is a zero crossing detector, ie channel 16 is the unvoicing.
*	detector the program keeps the strongest 3 of the peak 
*	channels when there is signal and the voicing is off
*	The detdata file has been converted to byte format.
*-----------------------------------------------------------------
*	This program computes the bandpass filter and
*	detector outputs for PCM input data files.
*	The file determined by the  command line filename 
*	is sequentially read for the names of speech files
*	to be converted. 
*-----------------------------------------------------------------

	program main

*-----------------------------------------------------------------
*	Parameter Definitions	
*-----------------------------------------------------------------
*
*	FILES 		!number of files to be converted  
*	BUFSIZE 	!maximum size of PCM datafile
*	NFILTERS	!number of filter-bank filters
*	DETS		!number of detector channels	
*	NPOINTS		!number of detector data points 
*	PCMRATE		!sampling rate of PCM data
*
*-----------------------------------------------------------------

	integer FILES  
	integer BUFSIZE 
	integer NFILTERS
	integer DETS	
	integer NPOINTS
	integer PCMRATE
	integer ask

	real detthreshold 
	real derivthreshold 
	real topendthreshold
	real lowendthreshold
	real max1
	real max2
	real max3
	real max4
	real max5
	real max6
	real temp

	integer n1
	integer n2
	integer n3
	integer n4
	integer n5
	integer n6
	integer ntemp

	parameter(FILES =10)
	parameter(BUFSIZE=18000)
	parameter(NFILTERS=16) 
	parameter(DETS=16)
	parameter(NPOINTS=90)
	parameter(PCMRATE=20000)

	parameter(detthreshold = 10.)
	parameter(derivthreshold = 0.00)
	parameter(topendthreshold =0.1)
	parameter(lowendthreshold = 0.0) 
	
	integer*2  shortpcm(BUFSIZE)	 
	integer*2  block(1000)	 
	real	pcm(BUFSIZE)

	real filterfreq(NFILTERS)
	real filtdata(BUFSIZE,NFILTERS)
	real qfilter(NFILTERS)
	real scfilter(DETS)
	real unvoice(BUFSIZE)
	real voice(BUFSIZE)
	real ampl(NPOINTS)
	integer zero(BUFSIZE)
	
	real tedfiltdata(NPOINTS,DETS)
	real redfiltdata(NPOINTS,DETS)
	byte detdata(NPOINTS,DETS)
	real realdata(NPOINTS,DETS)
	real keep(BUFSIZE)
	real g(16)
	real increment

	character*50 listfile	
	character*50 datafile	
	character*70 outdata

	logical oldfile
	integer num
	integer iargc

	integer statb(12)
	real maxpcm
	integer NDATAPOINTS
	integer npcmperpoint
	integer icrit
	icrit=20

*----------------------------------------------------------------
*	filterbank parameters
*-----------------------------------------------------------------


	filterfreq(1)=283.
	filterfreq(2)=403.
	filterfreq(3)=533.
	filterfreq(4)=677.
	filterfreq(5)=834.
	filterfreq(6)=1005.
	filterfreq(7)=1192.
	filterfreq(8)=1396.
	filterfreq(9)=1620.
	filterfreq(10)=1864.
	filterfreq(11)=2131.
	filterfreq(12)=2423.
	filterfreq(13)=2743.
	filterfreq(14)=3092.
	filterfreq(15)=3474.
	filterfreq(16)=3891.

*	this is normally 1.0 for 32 filters
	s=3.33
*	s=1.67 makes bandwidth 300 cycles	 the usual setting
	qfilter(1)=s*0.57
	qfilter(2)=s*0.81
	qfilter(3)=s*1.07
	qfilter(4)=s*1.35
	qfilter(5)=s*1.67
	qfilter(6)=s*2.00
	qfilter(7)=s*2.38
	qfilter(8)=s*2.79
	qfilter(9)=s*3.24
	qfilter(10)=s*3.72
	qfilter(11)=s*4.26
	qfilter(12)=s*4.85
	qfilter(13)=s*5.49
	qfilter(14)=s*6.18
	qfilter(15)=s*6.95
	qfilter(16)=s*7.78

	npcmperpoint=BUFSIZE/NPOINTS

*----------------------------------------------------------------
*	read in the filter channel gains from the file "gains"
*-----------------------------------------------------------------

	inquire(file='gains',exist=oldfile)
        if (oldfile) then
          open(unit=9,file='gains',status='old')
        else
          stop 'File "gains" does not exist.  Program stopped.'
        end if

	
	do 33 n=1,NFILTERS
	read(9,*)g(n)
33	continue
	close(9)

*----------------------------------------------------------------
*	Open the detectordata output file:modified for BYTE data	
*-----------------------------------------------------------------

	inquire(file='detdata',exist=oldfile)
	if (oldfile) then
	stop 'File "detdata" already exists.  Program stopped.'
*	 open(unit=9,file='detdata',status='old',access='direct',
*     :          form='unformatted',recl=DETS*NPOINTS)
	else
	open(unit=9,file='detdata',status='new',access='direct',
     :		form='unformatted',recl=DETS*NPOINTS)
	end if
*----------------------------------------------------------------
*	Get name of file containing data file names
*-----------------------------------------------------------------

	num = iargc()
	if (num.ne.0) then
		call getarg(1,listfile)	
	else
		print*,'No ASCII file name was given.'
		print*,'Please enter one now:'
		read *,listfile
	end if

	inquire (file=listfile,exist=oldfile)
	if (oldfile) then
		open(unit=7,file=listfile,status='old')
	else
		stop 'ASCII file does not exist.  Program stopped.'
	end if

*----------------------------------------------------------------
*       Loop over the speech data files
*-----------------------------------------------------------------
 
	do 100 nfile=1,FILES

*	Initialize the pcm arrays to 0

	do 400 ndata=1,BUFSIZE

		shortpcm(ndata)=0.
		pcm(ndata)=0.00001

400	continue

*	Initialize detdata array

	do 4440 ndet=1,DETS
	do 4450 npoint=1,NPOINTS

	   detdata(npoint,ndet)=0

4450	continue
4440	continue 
*------------------------------------------------------------
*	Read the data: crude blocking implemented 
*------------------------------------------------------------

	read(7,'(A)') datafile
	print*,'reading ',datafile

	call stat(datafile,statb)
	if(statb(8)/2.lt.BUFSIZE) then 
		nrecord = statb(8)/2 
	else
		nrecord = BUFSIZE
	endif 

	NDATAPOINTS=int ((real (nrecord)/BUFSIZE)*NPOINTS)
	print*,'nrecord=',nrecord
	print*,'datapoints=',NDATAPOINTS

*------------------------------------------------------------
	inquire (file=datafile,exist=oldfile)
        if (oldfile) then
          open(unit=8,file=datafile,status='old',form='unformatted',
     :    access='direct',recl=2000)
	else
          stop 'A data file does not exist.  Program stopped.'
        end if
*------------------------------------------------------------

	do 300 ndata=1,nrecord/1000
	read(8,rec=ndata) block

	do 305 mdata=1,1000
	shortpcm(((ndata-1)*1000)+mdata) = block(mdata)
305	continue

300	continue
	close(8) 		

	open(unit=8,file=datafile,status='old',form='unformatted',
     :    access='direct',recl=2)

	do 310 ndata=((nrecord/1000)*1000)+1,nrecord
	read(8,rec=ndata) shortpcm(ndata) 

310	continue
	close(8) 		

*-----------------------------------------------------------------
*	Normalize the amplitude
*-----------------------------------------------------------------

	maxpcm=0.
	do 500 ndata=1,BUFSIZE
	
		if(shortpcm(ndata).gt.maxpcm) then
			maxpcm = shortpcm(ndata)
		endif	
	
500	continue

	do 600 ndata=1,BUFSIZE

		pcm(ndata)=32767*shortpcm(ndata)/maxpcm
		if(pcm(ndata).eq.0.) pcm(ndata)=.00001

600	continue


	ftau = .010		
	delt=1.0/PCMRATE

*-----------------------------------------------------------------
*	short-time amplitude computation	
*-----------------------------------------------------------------

*	z=0.0
	do 333 mm=1,NPOINTS

*	This initialization has just been added.  It was done two
*	lines above (previously), so it was not reset for each point

	z=0.0

	do 332 nn=1,npcmperpoint
	npoint=nn+(mm-1)*npcmperpoint
	z=.99*z+abs(pcm(npoint))
332	continue
*	print*,'ampl(mm),mm ',mm,z/npcmperpoint
333	ampl(mm)=z/npcmperpoint
	
*-----------------------------------------------------------------
*	Filter-bank execution 
*-----------------------------------------------------------------

	do 700 nfilter=1,NFILTERS

*	filter 

	freq=filterfreq(nfilter)
	q=qfilter(nfilter)

	alpha = (1.-exp(-6.2832*freq*delt/q))/
     :   (1.+exp(-6.2832*freq*delt/q))
	gamma = 2.*alpha*sin(6.2832*freq*delt)
	beta= 2.*(1.-cos(6.2832*freq*delt))
	a=gamma/(1.+alpha)
	b=(2.-beta)/(1.+alpha)
	c=(alpha-1.)/(alpha+1.)
	filtdata(1,nfilter)=0.
	filtdata(2,nfilter)=0.
	do 10 n=3,BUFSIZE
	filtdata(n,nfilter)=a*pcm(n-1)+
     :     b*filtdata((n-1),nfilter)+c*filtdata((n-2),nfilter)
10	continue

*	rectify and integrate

	dum=exp(-delt/ftau)
	dnorm=1.-dum
	filtdata(1,nfilter)=0
	do 20 n=2,BUFSIZE
	filtdata(n,nfilter)=dum*filtdata(n-1,nfilter)
     :     +dnorm*(abs(filtdata(n,nfilter)))
20	continue
	
700	continue



*-----------------------------------------------------------------
*	Quantizer  
*-----------------------------------------------------------------

*	Average the filtered PCM data in one NPOINT time interval   


	do 1200 nfilt=1,NFILTERS
	do 1210 npoint=1,NPOINTS

		redfiltdata(npoint,nfilt)=0.

	do 1220 naverage=1,npcmperpoint
		
		redfiltdata(npoint,nfilt)=redfiltdata(npoint,nfilt)+
     :   	  filtdata((npoint-1)*npcmperpoint+naverage,nfilt)

1220	continue
		
		redfiltdata(npoint,nfilt)=
     :	      	redfiltdata(npoint,nfilt)/npcmperpoint
	

1210	continue
1200	continue
		
*------------------------------------------------------------
*	2nd ln deriv test for internal peaks; logic test for end	
*------------------------------------------------------------

1300	continue
	do 1302 nfilt=1,16
	do 1301 i=1,NPOINTS
	tedfiltdata(i,nfilt)=redfiltdata(i,nfilt)*g(nfilt)
1301	continue

*	1301	if(i.eq.icrit) write(*,*)nfilt,tedfiltdata(i,nfilt*2)

1302	continue

	do 1250 npoint=1,NPOINTS
	
	xx=((tedfiltdata(npoint,1)-tedfiltdata(npoint,2))
     :	 /tedfiltdata(npoint,1))-lowendthreshold
		
		realdata(npoint,1)=xx


*---------------------------------------------------------------
	do 1260 nfilt=2,NFILTERS-1

	x1=tedfiltdata(npoint,nfilt-1)
	x2=tedfiltdata(npoint,nfilt)
	x3=tedfiltdata(npoint,nfilt+1)

*---------------------------------------------------------------
	deriv=(x2-0.5*(x1+x3))/x2
	realdata(npoint,nfilt)=deriv
*	if(npoint.eq.icrit)write(*,*)nfilt,deriv
*---------------------------------------------------------------

1260	continue

1250	continue

*-----------------------------------------------------------------
*	make channel 16 for unvoiced speech
*-----------------------------------------------------------------

	do 5001 i=1,BUFSIZE-1
	zero(i)=0
	if(pcm(i+1).gt.10..and.pcm(i).lt.0.) zero(i)=1
5001	continue

	taureal1=100.
	beta=exp(-0.00005*taureal1)
	alpha=1.-beta
	voice(1)=0.
	unvoice(1)=0.
	do 5005 i=2,BUFSIZE
	unvoice(i)=0.
	keep(i)=0.
	voice(i)=beta*voice(i-1)+alpha*100.*zero(i)
	if(voice(i).gt.10.)unvoice(i)=1.
	if(voice(i).gt.1.0) keep(i)=1.0
5005	continue	

	do 5010 i=1,NPOINTS
	detdata(i,NFILTERS)=unvoice((BUFSIZE/NPOINTS)*i)
*	detdata(i,16)=keep(BUFSIZE/NPOINTS)*i)
5010	continue

*--------------------------------------------------------------
	do 9999 npoint=1,NDATAPOINTS
*	select the six maximum outputs in channels 1-15
	max1=-10.
	max2=-10.	
	max3=-10.	
	max4=-10.	
	max5=-10.	
	max6=-10.	
	n1=0
	n2=0
	n3=0
	n4=0
	n5=0
	n6=0

	do 777 ndet=1,NFILTERS-1
	detdata(npoint,ndet)=0
	if(realdata(npoint,ndet).gt.max6)then
	max6=realdata(npoint,ndet)
	n6=ndet
	endif

	if(max6.gt.max5)then
	temp=max6
	max6=max5
	max5=temp
	ntemp=n6
	n6=n5
	n5=ntemp
	endif

	if(max5.gt.max4)then
	temp=max5
	max5=max4
	max4=temp
	ntemp=n5
	n5=n4
	n4=ntemp
	endif

	if(max4.gt.max3)then
	temp=max4
	max4=max3
	max3=temp
	ntemp=n4
	n4=n3
	n3=ntemp
	endif

	if(max3.gt.max2)then
	temp=max3
	max3=max2
	max2=temp
	ntemp=n3
	n3=n2
	n2=ntemp
	endif

	if(max2.gt.max1)then
	temp=max2
	max2=max1
	max1=temp
	ntemp=n2
	n2=n1
	n1=ntemp
	endif



777	continue	
*	if((ampl(npoint).gt.100.0).or.2*detdata(npoint,NFILTERS).gt.1) then
	detdata(npoint,n1)=1
	detdata(npoint,n2)=1
	detdata(npoint,n3)=1
*	if(detdata(npoint,32)*2.lt.1) detdata(npoint,n4)=1  
*	if(detdata(npoint,32)*2.lt.1) detdata(npoint,n5)=1
	detdata(npoint,n6)=0
*	endif

9999	continue	
	
*-----------------------------------------------------------------
*	Write detector ouputs to file	
*-----------------------------------------------------------------

	write(9,rec=nfile) detdata

*	end of loop over files
100	continue 
	
	close(7)
 	close(9)

	stop
	end

