
*	This is the fortran version of the prelearn 
* 	program. The detector outputs are read in,
*	they are convolved with the delay lines,
*	and their outputs are saved in a .delay file
*	and the qdata file to be used in a subsequent
*	learning program.
*	
*	Modified for BYTE detdata and qdata. The value of qscale
*	should be set to 255/(10*taurecognition)
	program main

	integer LEARNPOINTS
	integer PCMRATE
	integer FILES
	integer WORDS
	integer DETS
	integer TAPS
	integer NPOINTS
	integer backconvolve 
	real tau
	real timestep
	real delaytau
	real delaysigma
	real qscale
	real qoffset
	

	parameter (LEARNPOINTS=19)
	parameter (PCMRATE=20000)
	parameter (FILES=10)
	parameter (WORDS=10)
	parameter (DETS=16)
	parameter (TAPS=10)
	parameter (NPOINTS=90)
	parameter (tau=.03)
	parameter (timestep=.01)
	parameter (delaytau=.04)
	parameter (delaysigma=.03)
	parameter (backconvolve=40)
	parameter (qscale=10.)
	parameter (qoffset=-128.)

	integer ndet
	integer ntap
	integer nfile
	integer npoint
	integer nback    
	integer temp

	character*50 detfile	
	character*50 delayfile	
	integer spword(FILES) 
	integer otherword(FILES) 
	integer anotherword(FILES) 
	integer njustify(FILES)
	real intdelaydata(LEARNPOINTS,TAPS,DETS)
	real integrated(NPOINTS)
	real delaydata(NPOINTS,TAPS,DETS)
	byte detdata(NPOINTS,DETS)
	real impulse(NPOINTS,TAPS)



	byte byteintdelaydata(LEARNPOINTS,TAPS,DETS)
	integer scaledvalue
	logical oldfile

*-------------------------------------------------------------
*	Print the parameters	
*
	
	print*,'PCMRATE=',PCMRATE
	print*,'FILES=',FILES
	print*,'WORDS=',WORDS
	print*,'DETS=',DETS
	print*,'TAPS=',TAPS
	print*,'NPOINTS=',NPOINTS
	print*,'tau=',tau
	print*,'timestep=',timestep
	print*,'delaytau=',delaytau
	print*,'delaysigma=',delaysigma
	print*,'backconvolve=',backconvolve

*-------------------------------------------------------------
*	Make the impulse response functions
*

	do 8010 npoint=1,NPOINTS
	impulse(npoint,5)=0.
	if(npoint-10.eq.int(4.*delaytau/timestep)) then
		impulse(npoint,5)=1.0
	endif
8010 	continue

	do 8020 ntap=1,TAPS
	if (ntap.ne.5) then
	do 8030 npoint=1,NPOINTS

	arg=(((npoint-10)*timestep)-((ntap-1)*delaytau))/
     :   (sqrt(real(abs((ntap-1)-4)))*delaysigma)
	impulse(npoint,ntap)=exp(-arg**2)

8030	continue
	endif
8020	continue
	
*	Normalize the area under the curves

	do 8040 ntap=1,TAPS

	sumimpulse=0.

	do 8050 npoint=1,NPOINTS

	sumimpulse = sumimpulse + impulse(npoint,ntap)

8050	continue

	do 8060 npoint=1,NPOINTS

	impulse(npoint,ntap) = impulse(npoint,ntap) * 10.0/sumimpulse

8060	continue

8040	continue

*-------------------------------------------------------------
*	Read in pdata 
*
	inquire (file='pdata',exist=oldfile)
        if (oldfile) then
	  open(unit=7,file='pdata',status='old')
        else
          stop '"pdata" file does not exist.  Program stopped.'
        end if

	read(7,*) (spword(nfile),
*     :  otherword(nfile),anotherword(nfile),
     :  njustify(nfile),nfile=1,FILES)
	close(7)

*-------------------------------------------------------------
*	Open the detdata file: BYTE format assumed
* #recl = DETS*NPOINTS

	inquire (file='detdata',exist=oldfile)
        if (oldfile) then
  	  open(unit=7,file='detdata',
     :    status='old',form='unformatted',
     :    access='direct',recl=DETS*NPOINTS)
        else
          stop '"detdata" file does not exist.  Program stopped.'
        end if

*-------------------------------------------------------------
*	Open the qdata output file 
* #recl = DETS*TAPS*LEARNPOINTS
*	BYTE FORMAT

	inquire(file='qdata',exist=oldfile)
        if (oldfile) then
        stop 'File "qdata" already exists.  Program stopped.'
*         open(unit=9,file='qdata',status='old',access='direct',
*     :   form='unformatted',recl=DETS*TAPS*LEARNPOINTS)
        else
	  open(unit=9,file='qdata',status='new',form='unformatted',
     :    access='direct',recl=DETS*TAPS*LEARNPOINTS)
        end if



*-------------------------------------------------------------
*	Open the delaydata output file
* #recl = 4*DETS*TAPS*NPOINTS

	print*,'Not saving delaydata'
*  	 open(unit=8,file='delaydata',
*     :  status='new',access='direct',
*     :  form='unformatted',recl=4*DETS*TAPS*NPOINTS)
	
*-------------------------------------------------------------
*	loop over the files
*
	do 400 nfile=1,FILES


	read(7,rec=nfile) detdata

	print*,'Doing segment ',nfile

*-------------------------------------------------------------
* 	Convolve the detector data 
*

	do 4310 ndet=1,DETS
	do 4320 ntap=1,TAPS
	do 4330 npoint=1,NPOINTS
	
	delaydata(npoint,ntap,ndet)=0.

4330	continue
4320	continue
4310	continue

	do 3340 nback=1,backconvolve
	do 3310 ndet=1,DETS
	do 3320 ntap=1,TAPS
	do 3330 npoint=1,NPOINTS


	if (nback.lt.npoint) then
		temp = detdata(npoint-nback,ndet)
	delaydata(npoint,ntap,ndet)=delaydata(npoint,ntap,ndet)+
     :		  temp*impulse(nback,ntap)

	endif

3330	continue	
3320	continue	
3310	continue	
3340	continue	

*-------------------------------------------------------------
* 	Integrate the detector data, simulating
*	input on the recognition units
*

	do 100 ndet=1,DETS
	do 105 ntap=1,TAPS

	integrated(1)=0.

	do 110 npoint=2,NPOINTS

	integrated(npoint) = (1.0-timestep/tau)*integrated(npoint-1) 
     :	  + timestep*delaydata(npoint,ntap,ndet) 

110	continue

	do 120 npoint=1,LEARNPOINTS-1

	intdelaydata(npoint,ntap,ndet)=
     :    integrated(((npoint)*NPOINTS)/(LEARNPOINTS-1))

120	continue

	intdelaydata(LEARNPOINTS,ntap,ndet)=
     :     integrated((njustify(nfile)/int(PCMRATE*timestep))+1)

105	continue
100	continue

*-------------------------------------------------------------
*	scale the qdata entry and compress it
*	scaledvalue must be done in two steps to assure correct 
*	rounding of negative integers

	do 410 ndet=1,DETS
	do 420 ntap=1,TAPS
	do 430 npoint=1,LEARNPOINTS

	scaledvalue=qscale*intdelaydata(npoint,ntap,ndet)
	scaledvalue=scaledvalue+qoffset

	if(scaledvalue.gt.127.) then
	print*,'exceeded limit; qdata=', scaledvalue 
	scaledvalue=127
	endif


	byteintdelaydata(npoint,ntap,ndet)=scaledvalue


430	continue
420	continue
410	continue

*-------------------------------------------------------------
* 	Write the qdata entry 
*

	write(9,rec=nfile)  byteintdelaydata 

*-------------------------------------------------------------
* 	Write the delay data entry 
*

*	write(8,rec=nfile) delaydata
	
*-------------------------------------------------------------
*	end of loop over files

400	continue

	close(7)
*	close(8)
	close(9)


	stop
	end


