*	This is the fortran version of the backend of the
*	recognition circuit. Either detector data or
*	delaydata can be used as input to the program.
*	The recognition unit outputs are written to a file
*	
	program main

	integer STARTDELAY
	integer FILES
	integer WORDS
	integer DETS
	integer TAPS
	integer NPOINTS
	integer backconvolve 
	real tau
	real timestep
	real delaytau
	real delaysigma
	

	parameter (STARTDELAY=0)
	parameter (FILES=10)
	parameter (WORDS=10)
	parameter (DETS=16)
	parameter (TAPS=10)
	parameter (NPOINTS=90)
	parameter (tau=.03)
	parameter (timestep=.01)
	parameter (delaytau=.04)
	parameter (delaysigma=.03)
	parameter (backconvolve=40)

	integer nword
	integer ndet
	integer ntap
	integer nfile
	integer npoint
	integer taplimit(WORDS)
	integer temp

	real uinput(NPOINTS,WORDS)
	real voutput(NPOINTS,WORDS)
	real delconn(WORDS,DETS,TAPS)
	real globinh(WORDS)
	real delaydata(NPOINTS,TAPS,DETS)
	byte detdata(NPOINTS,DETS)
	real impulse(NPOINTS,TAPS)
	real delayin

	logical oldfile

*-------------------------------------------------------------
*	Print the parameters	
*
	
	print*,'STARTDELAY=',STARTDELAY
	print*,'FILES=',FILES
	print*,'WORDS=',WORDS
	print*,'DETS=',DETS
	print*,'TAPS=',TAPS
	print*,'NPOINTS=',NPOINTS
	print*,'tau=',tau
	print*,'timestep=',timestep
	print*,'delaytau=',delaytau
	print*,'delaysigma=',delaysigma
	print*,'backconvolve=',backconvolve

*------------------------------------------------------------
*	load the connections
*
	inquire (file='connections',exist=oldfile)
        if (oldfile) then
          open(unit=7,file='connections',status='old')
        else
          stop '"connections" file does not exist.  Program stopped.'
        end if

	do 100 nword=1,WORDS
	do 110 ndet=1,DETS
	do 120 ntap=1,TAPS

	read (7,*) delconn(nword,ndet,ntap)

120	continue
110	continue
100	continue

	do 130 nword=1,WORDS

	read (7,*) globinh(nword)

130	continue

	close(7)

*-------------------------------------------------------------
*	Limit the longest delay line taps per word 
*	according to the mean word length.
*
	taplimit(1)= 9 
	taplimit(2)= 7 
	taplimit(3)= 6 
	taplimit(4)= 8 
	taplimit(5)= 7 
	taplimit(6)= 8 
	taplimit(7)= 9 
	taplimit(8)= 9 
	taplimit(9)= 7 
	taplimit(10)= 8  

*-------------------------------------------------------------
*	Create the gain lookup table
*
	call gaincreate()


*-------------------------------------------------------------
*	Read in a gain lookup table
*
	call gainrel()


*-------------------------------------------------------------
*	Make the impulse response functions
*

	do 8010 npoint=1,NPOINTS
	impulse(npoint,5)=0.
	if(npoint-10.eq.int(4.*delaytau/timestep)) then
		impulse(npoint,5)=1.0
	endif
8010 	continue

	do 8020 ntap=1,TAPS
	if (ntap.ne.5) then
	do 8030 npoint=1,NPOINTS

	arg=(((npoint-10)*timestep)-((ntap-1)*delaytau))/
     :   (sqrt(real(abs((ntap-1)-4)))*delaysigma)

	impulse(npoint,ntap)=exp(-arg**2)


8030	continue
	endif
8020	continue
	
*	Normalize the area under the curves

	do 8040 ntap=1,TAPS

	sumimpulse=0.

	do 8050 npoint=1,NPOINTS

	sumimpulse = sumimpulse + impulse(npoint,ntap)

8050	continue

	do 8060 npoint=1,NPOINTS

	impulse(npoint,ntap) = impulse(npoint,ntap) * 10.0/sumimpulse

8060	continue

8040	continue

*-------------------------------------------------------------
*	Open the detdata file
* #recl = DETS*NPOINTS BYTE FORMAT

	if(STARTDELAY.ne.1) then	
	inquire(file='detdata',exist=oldfile)
        if (oldfile) then
	  open(unit=7,file='detdata',
     :    status='old',form='unformatted',
     :    access='direct',recl=DETS*NPOINTS)
	else
          stop 'File "detdata" does not exist.  Program stopped.'
	endif
	endif

*-------------------------------------------------------------
*	Open the delaydata file
* #recl = 4*DETS*TAPS*NPOINTS

	if(STARTDELAY.eq.1) then

	inquire(file='delaydata',exist=oldfile) 
	if (oldfile) then   
          open(unit=7,file='delaydata',
     :    status='old',form='unformatted',
     :    access='direct',recl=4*DETS*TAPS*NPOINTS)
        else
          stop 'File "delaydata" does not exist.  Program stopped.'
        endif

	endif

*-------------------------------------------------------------
*	Open the output data file
*#recl = 4*NPOINTS*WORDS

	inquire(file='recdata',exist=oldfile)
        if (oldfile) then
	  stop 'File "recdata" already exists.  Program stopped.' 
	else
	  open(unit=8,file='recdata',
     :    status='new',form='unformatted',
     :    access='direct',recl=4*NPOINTS*WORDS)
	endif

*	open(unit=9,file='udata',
*     :  status='new',form='unformatted',
*     :  access='direct',recl=4*NPOINTS*WORDS)

*-------------------------------------------------------------
*-------------------------------------------------------------
*	loop over the files
*
	do 400 nfile=1,FILES

*--------------------------------------------------------
	if(STARTDELAY.ne.1) then

	read(7,rec=nfile) detdata
	print*,'Doing segment ',nfile
*
* 	Convolve the detector data 
*
	do 4310 ndet=1,DETS
	do 4320 ntap=1,TAPS
	do 4330 npoint=1,NPOINTS
	
	delaydata(npoint,ntap,ndet)=0.

4330	continue
4320	continue
4310	continue

	do 3340 nback=1,backconvolve
	do 3310 ndet=1,DETS
	do 3320 ntap=1,TAPS
	do 3330 npoint=1,NPOINTS

	if (nback.lt.npoint) then
	
	temp = detdata(npoint-nback,ndet)
	delaydata(npoint,ntap,ndet)=delaydata(npoint,ntap,ndet)+
     :		  temp*impulse(nback,ntap)

	endif

3330	continue	
3320	continue	
3310	continue	
3340	continue	

	endif
*--------------------------------------------------------
*	Read the integrated delay data if appropriate
*	

	if(STARTDELAY.eq.1) then

	read(7,rec=nfile) delaydata
	print*,'Doing segment ',nfile

	endif
*-------------------------------------------------------------
* 	Compute the outputs  
*	of the recognition units.
*

	do 790 nword=1,WORDS

	uinput(1,nword)=globinh(nword)*tau	

	voutput(1,nword) = gain(uinput(1,nword))

	do 800 npoint=2,NPOINTS
	
	delayin = 0.
	do 820  ndet=1,DETS
	do 830  ntap=1,taplimit(nword)

	delayin = delayin +delaydata(npoint,ntap,ndet)*
     :   delconn(nword,ndet,ntap)


830	continue
820	continue


	uinput(npoint,nword)=uinput(npoint-1,nword)+ timestep *
     :   (-uinput(npoint-1,nword)/tau + delayin + globinh(nword))  

	voutput(npoint,nword) = gain(uinput(npoint,nword))

800	continue
790	continue


*-------------------------------------------------------------
* 	Write recognition unit outputs to the file
*

*	write(9,rec=nfile) uinput
	write(8,rec=nfile) voutput
	

	
*-------------------------------------------------------------
*	end of loop over files

400	continue

	close(7)
	close(8)
*	close(9)


	stop
	end

*-------------------------------------------------------------
*	Subroutine to create the input-ouput relation	
*

	subroutine gaincreate()

	real iorv(2001)
	real iorr(2001)
	real iowidth

	parameter (iowidth=.03)	
	
	common/lookup/iorv,iorr
	
	do 100 n=1,2001

	
	iorv(n)=(1.0/2.0)*(1+tanh((n-1001.0)/(iowidth*1000)))

100	continue

	do 200 n=1,2000

	iorr(n)=iorv(n+1)-iorv(n)
	
200	continue

*-------------------------------------------------------------
*	Store the tables in a file so that other programs can
*	use the same values.

2200	format(F11.9)
	open(unit=7,file='gaindata')
	write(7,2200)iowidth
	write(7,2200)(iorv(n),n=1,2001)
	write(7,2200)(iorr(n),n=1,2001)
	close(7)

	return
	end

*-------------------------------------------------------------
*	Subroutine to create the input-ouput relation	
*
	subroutine gainrel()
	real iorv(2001)
	real iorr(2001)
	real iowidth

	common/lookup/iorv,iorr

*-------------------------------------------------------------
*	Read the tables in from a file named gaindata 
*	Tables created using gaincreate(). The io_width
*	used the table is the first number read in. 

2300	format(F11.9)
	open(unit=7,file='gaindata',status='old')
	read(7,2300)iowidth
	print*,'reading gaindata file, iowidth=',iowidth
	read(7,2300)(iorv(n),n=1,2001)
	read(7,2300)(iorr(n),n=1,2001)
	close(7)

	return
	end

*-------------------------------------------------------------
*	Subroutine to create the input-ouput relation	
*

	real function gain(x)
	real x


	integer xi
	real xr
	real iorv(2001)
	real iorr(2001)
	real iowidth

	parameter (iowidth=0.03)

	common/lookup/iorv,iorr

	if(x.le.-1.0) then

		gain=iorv(1)

	elseif(x.ge.1.0) then

		gain=iorv(2001)
	
	else

		xi=int(x*1000.0)
		xr=(x*1000.0)-xi


	  if(x.ge.0.0) then

	    gain= iorv(xi+1001)+(xr*iorr(xi+1001))

	  elseif(x.lt.0.0) then

	    gain= iorv(xi+1000)+((1+xr)*iorr(xi+1000))

 	  endif

	endif	

	if (gain.eq.1.0) then
		gain = 0.5*(1+tanh(x/iowidth))
	endif

	end

