/*
 * tststats.c reads in the recognition unit outputs of N_points
 * long from the unformatted data file entered on the command
 * line. The pdata file is used to supply the data on the
 * correct unit for the given segment. If the correct recognition
 * unit is only driven on below the 0.5 level, or an
 * incorrect unit is driven above 0.5, the segment is flagged
 * and an entry is written in the file badsegments. Detailed
 * errors are written in the file testdata. At the end
 * of the analysis, percent correct statistics are appended
 * to the end of the badsegments file and printed on the terminal.
 */

#include <stdio.h>
#include <string.h>

#define N_words	10
#define N_points 90
#define N_files	10

float threshold=0.50;

float v_recog[N_words][N_points];

main(argc,argv)
int argc;
char *argv[];
{
	int i;

	int n_word,n_step,n_file;
	int n_read;
	int n_show;
	int sp_word;
	int n_justify;
	int fd; 
	int correct_wrong;
	int incorrect_wrong;
	int bad_segment;
	int bad_flag;
	float maxv;

	FILE *fp,*fp1,*fp2;

	char message[80];
	char heading[80];

	printf("Threshold=%f\n",threshold);	

	fd=open(*++argv,0);
	fp=fopen("pdata","r");	
	fp1=fopen("testdata","w");
	fp2=fopen("badsegments","w");


	for(n_file=0;n_file<N_files;n_file++){

	n_read=read(fd,v_recog,4*N_words*N_points);

	fscanf(fp,"%d",&sp_word);
	fscanf(fp,"%d",&n_justify);

	bad_flag=0;	
	for(n_word=0;n_word<N_words;n_word++){
	maxv=0.;
	for(n_step=0;n_step<N_points;n_step++){
		if(v_recog[n_word][n_step]>maxv)
		    maxv=v_recog[n_word][n_step];
	}/*loop on n_step*/
	
	if((n_word==sp_word)&&(maxv<threshold)){
		correct_wrong++;
		bad_flag=1;
		fprintf(fp1,"%d file: Sp_word=%d: Error: Correct =%f\n",
			n_file+1,sp_word,maxv);
	}
	if ((n_word!=sp_word)&&(maxv>=threshold)) {
		incorrect_wrong++;	
		bad_flag=1;
		fprintf(fp1,"%d file: Sp_word=%d: Error: Incorrect word(%d) = %f\n",
			n_file+1,sp_word,n_word,maxv);
	}
	
	}/*loop on n_word*/
	if(bad_flag==1){
		bad_segment++;
		fprintf(fp2,"Segment %d:Bad recognition\
 of %d\n",n_file+1,sp_word); 
	}
	}/*loop on n_file*/
	printf("Number Correct:%d out of %d files;\
	Percent Correct:%f\n",
	(N_files-bad_segment),N_files,
	100.*(N_files-bad_segment)/N_files);
	printf("Number low value of correct unit: %d\
  Percent:%f\n",
		correct_wrong,100.*correct_wrong/N_files);
	printf("Number high value of incorrect unit: %d\
  Percent:%f\n",
		incorrect_wrong,100.*incorrect_wrong/
		((N_words-1)*N_files));
	fprintf(fp2,"Number Correct:%d out of %d files;\
	Percent Correct:%f\n",
	(N_files-bad_segment),N_files,
	100.*(N_files-bad_segment)/N_files);
	fprintf(fp2,"Number low value of correct unit: %d\
  Percent:%f\n",
		correct_wrong,100.*correct_wrong/N_files);
	fprintf(fp2,"Number high value of incorrect unit: %d\
  Percent:%f\n",
		incorrect_wrong,100.*incorrect_wrong/
		((N_words-1)*N_files));
	close(fd);
	close(fp);
	close(fp1);
	close(fp2);
}



